@@ -45,15 +45,18 @@ def get_R_HOME(force_exec=False):
       rhome = win32api.RegQueryValueEx( hkey, "InstallPath" )[0]
       win32api.RegCloseKey( hkey )
     except:
-      print "Unable to determine R version from the registry."
+      print "Unable to determine R version from the registry. Trying another method."
       pass  # failover to exec method
 
   # step 3: Try executing R to find out
-  if not rhome:
-    stat, rhome = getstatusoutput('R RHOME')
+  if not rhome or not os.path.exists(rhome):
+    if sys.platform == 'win32':
+      stat, rhome = getstatusoutput('R RHOME')
+    else:
+      stat, rhome = getstatusoutput('R RHOME | tail -1')
     if stat or len(rhome)<=0:
       raise RuntimeExecError("Couldn't execute the R interpreter.\n"
-                               "Check whether R is in the execution path.")
+                             "Check whether R is in the execution path.")
     
   rhome = rhome.strip()
   return rhome
@@ -67,7 +70,7 @@ def get_R_VERSION(RHOME=None, force_exec=False):
   rver = None
   
   # step 1: (Windows) Try the registry
-  if sys.platform == 'win32' and not force_exec:
+  if (not RHOME) and (sys.platform == 'win32') and (not force_exec):
     try:
       # Use the registry to find where R.dll is
       import win32api, win32con
@@ -80,12 +83,13 @@ def get_R_VERSION(RHOME=None, force_exec=False):
       rver =  win32api.RegQueryValueEx( hkey, "Current Version" )[0]
       win32api.RegCloseKey( hkey )
     except:
-      print "Unable to determine R version from the registry."
+      print "Unable to determine R version from the registry." + \
+            " Trying another method."
       pass  # failover to exec method
 
   # step 2: Try executing R to determine the version
   if not rver:
-    if RHOME is None:
+    if (not RHOME) or (not os.path.exists(RHOME)):
       RHOME = get_R_HOME(force_exec)
 
     rexec = os.path.join(RHOME, 'bin', 'R')
@@ -108,3 +112,33 @@ def get_R_VERSION_CODE( verstr=None ):
     rver = "%1d%02d%1d" % (rmajor, rminor, rpatch)
     return rver
 
+def get_R_USER():
+  RUSER = os.getenv("R_USER")
+  if not os.path.exists(str(RUSER)):
+    RUSER = os.getenv("HOME")
+  if not os.path.exists(str(RUSER)) and sys.platform == 'win32':
+    RUSER = os.getenv("HOMEDRIVE") + os.getenv("HOMEPATH")
+  if not os.path.exists(str(RUSER)):
+    RUSER=os.getcwd()
+  return RUSER
+
+
+def get_PYTHON_DIR():
+  ver = "%d.%d" % sys.version_info[0:2]
+  try: 
+    import win32api, win32con
+    hkey = win32api.RegOpenKeyEx(
+      win32con.HKEY_LOCAL_MACHINE,
+      "Software\\Python\\PythonCore\\%s\\InstallPath" % ver,
+      0, win32con.KEY_QUERY_VALUE )
+    # get the base directory
+    PYTHON_DIR = win32api.RegQueryValueEx( hkey, "")[0]
+    win32api.RegCloseKey( hkey )
+  except:
+    raise RuntimeError("Unable to determine Python install location from the registry")
+
+  if not os.path.exists(PYTHON_DIR):
+    raise RuntimeError("Python install location from the registry does not exist: `%s'"
+                       % PYTHON_DIR )
+    
+  return PYTHON_DIR
