@@ -1,25 +1,43 @@
 /*
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- *
- *  Implementation of the module '_rpy' and the 'Robj' type.
- *
- *  $Id: rpymodule.c,v 1.31 2005/07/28 14:23:33 warnes Exp $
- *
+ * $Id: rpymodule.c 299 2006-03-22 22:13:54Z warnes $
+ * Implementation of the module '_rpy' and the 'Robj' type.
  */
 
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ *
+ * The Original Code is the RPy python module.
+ *
+ * The Initial Developer of the Original Code is Walter Moreira.
+ * Portions created by the Initial Developer are Copyright (C) 2002
+ * the Initial Developer. All Rights Reserved.
+ *
+ * Contributor(s):
+ *    Gregory R. Warnes <greg@warnes.net> (Maintainer)
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #include "RPy.h"
 
@@ -37,10 +55,14 @@ static int use_numeric=0;
 
 
 /* Local function definitions */
-DL_EXPORT(void) INIT_RPY(void);                     /* Module initializer */
-static PyObject *r_init(PyObject *self, PyObject *args); /* Class initializer */
-static PyObject *r_cleanup(void);                    /* Clean up R & release resources */
+DL_EXPORT(void) INIT_RPY(void);           /* Module initializer */
+static PyObject *r_init(PyObject *self,   /* Class initializer */
+                        PyObject *args);
+static PyObject *r_cleanup(void);         /* Clean up R & release resources */
 
+#ifdef _WIN32
+static void init_embedded_win32( void );
+#endif
 
 /* Global objects */
 static SEXP get_item;
@@ -53,6 +75,11 @@ static int default_mode;
 static PyObject *r_lock;
 PyObject *RPyExc_Exception;
 
+static char RHOME[BUFSIZ];
+static char RVERSION[BUFSIZ];
+static char RVER[BUFSIZ];
+static char RUSER[BUFSIZ];
+
 /* Global interpreter */
 PyInterpreterState *my_interp;
 
@@ -227,8 +254,8 @@ seq_to_R(PyObject *obj)
     case STRING_T:
       robj = AS_CHARACTER(robj);
       break;
-    case ROBJ_T: /* We should never see an Robj here */
-      robj = NULL;
+    default:;
+      /* Otherwise, it's either an ANY_T or ROBJ_T - we want ANY */
     }
   
   UNPROTECT(1);
@@ -398,8 +425,8 @@ to_Robj(PyObject *obj)
       SET_STRING_ELT(robj, 0, COPY_TO_USER_STRING(PyString_AsString(obj)));
     }
 #ifdef WITH_NUMERIC
-  //  else if (use_numeric && PyArray_Check(obj)) 
-  else if (PyArray_Check(obj)) 
+  //else if (PyArray_Check(obj)) 
+  else if (use_numeric && PyArray_Check(obj)) 
     {
       PROTECT(robj = to_Rarray(obj));
     }
@@ -875,6 +902,8 @@ dotter(char *s)
   char *r, *res;
   int l;
 
+  if (!s)
+    return NULL;                /* assume prev PyString_AsString has failed */
   l = strlen(s);
   r = (char *)PyMem_Malloc(l+1);
   if (!r) {
@@ -900,6 +929,7 @@ make_kwds(int lkwds, PyObject *kwds, SEXP *e)
   char *s;
   int i;
   PyObject *citems=NULL, *it;
+  PyObject *kwname;
 
   if (kwds) {
     citems = PyMapping_Items(kwds);
@@ -907,13 +937,22 @@ make_kwds(int lkwds, PyObject *kwds, SEXP *e)
 
   for (i=0; i<lkwds; i++) {
     it = PySequence_GetItem(citems, i);
+    if (!it)
+      goto fail;
     r = to_Robj(PyTuple_GetItem(it, 1));
     Py_DECREF(it);
     if (!r)
       goto fail;
 
     SETCAR(*e, r);
-    s = dotter(PyString_AsString(PyTuple_GetItem(it, 0)));
+    kwname = PyTuple_GetItem(it, 0);
+    if (!kwname)
+      goto fail;
+    if (!PyString_Check(kwname)) {
+      PyErr_SetString(PyExc_TypeError, "keywords must be strings");
+      goto fail;
+    }
+    s = dotter(PyString_AsString(kwname));
     if (!s)
       goto fail;
 
@@ -1289,7 +1328,7 @@ static PySequenceMethods Robj_as_sequence = {
 /* The 'Robj' table. When compiled under Python 2.2, the type 'Robj'
    is subclassable. */
 
-#if defined(PRE_2_2) || defined(_WIN32)
+#ifdef PRE_2_2
 static PyObject *
 Robj_getattr(RobjObject *self, char *name)
 {
@@ -1538,7 +1577,8 @@ r_init(PyObject *self, PyObject *args)
   use_numeric = i;
 
 #ifdef WITH_NUMERIC
-  init_numeric();
+  if(use_numeric)
+    init_numeric();
 #endif
   
   if(first==1)
@@ -1600,70 +1640,26 @@ init_embedded_win32( void ) {
   structRstart rp;
   Rstart Rp = &rp;
   char Rversion[25];
-  static char RUser[MAX_PATH], RHome[MAX_PATH]; /* BR */
-  char *p;
+  int index;
 
 
   snprintf( Rversion, 25, "%s.%s", R_MAJOR, R_MINOR );
   if( strcmp( getDLLVersion(), Rversion ) != 0 ) { 
-    PyErr_SetString( PyExc_ImportError, "R.DLL version does not match" ); 
+    PyErr_SetString( PyExc_ImportError, "R.DLL version does not match" );
+    return;
   } 
 
   R_DefParams(Rp);
 
-  /* first, try process-local environment space (CRT) */
-  if ( getenv( "R_HOME" ) ) {
-    strcpy( RHome, getenv("R_HOME") );
-  }
-  else {
-    /* get variable from process-local environment space (Windows API) */
-    if ( GetEnvironmentVariable( "R_HOME", RHome, sizeof( RHome ) ) == 0) {
-
-      /* not found, look in the registry */
-      LONG rc;
-      HKEY hkey;
-      DWORD keytype = REG_SZ;
-      DWORD cbData = sizeof( RHome );
-      rc = RegOpenKeyEx (HKEY_LOCAL_MACHINE, "Software\\R-core\\R", 0, 
-                        KEY_READ, &hkey);
-
-      if (rc == ERROR_SUCCESS) {
-       rc = RegQueryValueEx (hkey, "InstallPath", 0, &keytype, 
-                             (LPBYTE)RHome, &cbData);
-       RegCloseKey (hkey);
-      }
-
-      if (rc == ERROR_SUCCESS) {
-        /* set R_HOME */
-       char *buf = (char *) malloc( (strlen( RHome ) + 8) * sizeof( char ));
-       strcpy( buf, "R_HOME=");
-       strcat( buf, RHome );
-       putenv( buf );
-      }
-      else {
-        /* not found, fall back to getRHOME() */
-        strcpy(RHome, getRHOME());
-      }
-    }
-  }
   /* set R_HOME */
-  Rp->rhome = RHome;
-  /*
-   * try R_USER then HOME then working directory
-   */
-  if (getenv("R_USER")) {
-    strcpy(RUser, getenv("R_USER")); /* BR */
-  } else if (getenv("HOME")) {
-      strcpy(RUser, getenv("HOME"));
-  } else if (getenv("HOMEDRIVE")) { /* BR */
-      strcpy(RUser, getenv("HOMEDRIVE"));
-      strcat(RUser, getenv("HOMEPATH"));
-  } else
-      GetCurrentDirectory(MAX_PATH, RUser);
-  p = RUser + (strlen(RUser) - 1); /* BR */
-  if (*p == '/' || *p == '\\') *p = '\0'; /* BR */
+  Rp->rhome = RHOME;
 
-  Rp->home = RUser;
+  index = strlen(RUSER) - 1;
+  
+  if (RUSER[index] == '/' || RUSER[index] == '\\')
+    RUSER[index] = '\0';
+
+  Rp->home = RUSER;
   Rp->CharacterMode = LinkDLL;
     
   Rp->ReadConsole = (blah1) RPy_ReadConsole;    // Matjaz
@@ -1705,20 +1701,30 @@ init_embedded_win32( void ) {
 }
 #endif
 
-
 /* Initialization function for the module */
 DL_EXPORT(void)
 INIT_RPY(void)
 {
-  static char RHome[] = R_HOME;
   PyObject *m, *d;
-  char *defaultargv[] = {"rpy", "-q", "--vanilla"};
   PyOS_sighandler_t old_int;
 #ifndef _WIN32
+  char *defaultargv[] = {"rpy", "-q", "--vanilla"};
   PyOS_sighandler_t old_usr1, old_usr2;
 #endif
   SEXP interact;
 
+  /* Get path and version information from environment */
+  strncpy(RHOME,    getenv("RPY_RHOME"),    BUFSIZ);
+  strncpy(RVERSION, getenv("RPY_RVERSION"), BUFSIZ);
+  strncpy(RVER,     getenv("RPY_RVER"),     BUFSIZ);
+  strncpy(RUSER,    getenv("RPY_RUSER"),    BUFSIZ);
+
+  if( !strlen(RHOME) || !strlen(RVERSION) || !strlen(RVER) || !strlen(RUSER))
+    {
+      PyErr_Format(RPyExc_Exception,
+                   "Unable to load R path or version information");
+      return;
+    }
 
   Robj_Type.ob_type = &PyType_Type;
 #if defined( _WIN32 ) && ! defined( PRE_2_2 )
@@ -1726,6 +1732,7 @@ INIT_RPY(void)
   Robj_Type.tp_alloc = PyType_GenericAlloc;
 #endif
 
+  //m = Py_InitModule(MacroQuote(RPY_SHNAME), rpy_methods);
   m = Py_InitModule(RPY_SHNAME, rpy_methods);
   d = PyModule_GetDict(m);
 
@@ -1745,20 +1752,6 @@ INIT_RPY(void)
 #ifdef _WIN32
   init_embedded_win32();
 #else
-  //     envvar    c-def
-  if ( strlen(RHome)>0 )
-    {
-      setenv("R_HOME", RHome, 1);
-    }
-  else
-    {
-      // TODO: put code here to Run R RHOME to get R_HOME value
-      fprintf(stderr, "Error setting R_HOME at build time.\n");
-      exit(-1);
-    } 
-
-
-
   Rf_initEmbeddedR( sizeof(defaultargv) / sizeof(defaultargv[0]),
                     defaultargv);
 #endif
@@ -1821,28 +1814,3 @@ INIT_RPY(void)
 }
 
 
-#ifndef _WIN32
-char *getRHOME(void)
-{
-  char *cmd = "R RHOME";
-  char *buf;
-  FILE *ptr;
-  int index;
-
-  buf = malloc(BUFSIZ);
-  
-  if ((ptr = popen(cmd, "r")) != NULL)
-    fgets(buf, BUFSIZ, ptr);
-  (void) pclose(ptr);
-
-  index = strlen(buf);
-  index--; 
-
-  // strip off trailing newline or CR
-  while( buf[index]=='\n' || buf[index]=='\r' )
-    buf[index--] = 0;
-  
-  return buf;
-}
-#endif
-
