@@ -3,6 +3,9 @@ import sys
 import unittest
 import random
 
+import rpy
+
+
 def run(module):
     try:
         unittest.main(module)
@@ -21,5 +24,6 @@ if __name__ == '__main__':
     
     for module in modules:
         print 'Testing:', module[5:-3]
+        rpy.set_default_mode(rpy.NO_DEFAULT)  # reset to base case
         name = module[:-3]
         run(name)
